<?php

namespace Blogtest\Blogtest\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Blogtest\Blogtest\Http\Requests\ArticlesRequest;
use Blogtest\Blogtest\Interfaces\ArticlesRepositoryInterface;
use Blogtest\Blogtest\Models\Articles;

/**
 * Resource controller class for articles.
 */
class ArticlesResourceController extends BaseController
{

    /**
     * Initialize articles resource controller.
     *
     * @param type ArticlesRepositoryInterface $articles
     *
     * @return null
     */
    public function __construct(ArticlesRepositoryInterface $articles)
    {
        parent::__construct();
        $this->repository = $articles;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Blogtest\Blogtest\Repositories\Criteria\ArticlesResourceCriteria::class);
    }

    /**
     * Display a list of articles.
     *
     * @return Response
     */
    public function index(ArticlesRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Blogtest\Blogtest\Repositories\Presenter\ArticlesPresenter::class)
                ->$function();
        }

        $articles = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('blogtest::articles.names'))
            ->view('blogtest::articles.index', true)
            ->data(compact('articles', 'view'))
            ->output();
    }

    /**
     * Display articles.
     *
     * @param Request $request
     * @param Model   $articles
     *
     * @return Response
     */
    public function show(ArticlesRequest $request, Articles $articles)
    {

        if ($articles->exists) {
            $view = 'blogtest::articles.show';
        } else {
            $view = 'blogtest::articles.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('blogtest::articles.name'))
            ->data(compact('articles'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new articles.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(ArticlesRequest $request)
    {

        $articles = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('blogtest::articles.name')) 
            ->view('blogtest::articles.create', true) 
            ->data(compact('articles'))
            ->output();
    }

    /**
     * Create new articles.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(ArticlesRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $articles                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('blogtest::articles.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('blogtest/articles/' . $articles->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/blogtest/articles'))
                ->redirect();
        }

    }

    /**
     * Show articles for editing.
     *
     * @param Request $request
     * @param Model   $articles
     *
     * @return Response
     */
    public function edit(ArticlesRequest $request, Articles $articles)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('blogtest::articles.name'))
            ->view('blogtest::articles.edit', true)
            ->data(compact('articles'))
            ->output();
    }

    /**
     * Update the articles.
     *
     * @param Request $request
     * @param Model   $articles
     *
     * @return Response
     */
    public function update(ArticlesRequest $request, Articles $articles)
    {
        try {
            $attributes = $request->all();

            $articles->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('blogtest::articles.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('blogtest/articles/' . $articles->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('blogtest/articles/' . $articles->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the articles.
     *
     * @param Model   $articles
     *
     * @return Response
     */
    public function destroy(ArticlesRequest $request, Articles $articles)
    {
        try {

            $articles->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('blogtest::articles.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('blogtest/articles/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('blogtest/articles/' . $articles->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple articles.
     *
     * @param Model   $articles
     *
     * @return Response
     */
    public function delete(ArticlesRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('blogtest::articles.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('blogtest/articles'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/blogtest/articles'))
                ->redirect();
        }

    }

    /**
     * Restore deleted articles.
     *
     * @param Model   $articles
     *
     * @return Response
     */
    public function restore(ArticlesRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('blogtest::articles.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/blogtest/articles'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/blogtest/articles/'))
                ->redirect();
        }

    }

}
